import { useState, useEffect, useRef, useCallback } from 'react';
import "./styles.sass";

const frequencyToNote = (frequency) => {
  const A4 = 440;
  const notes = ['C', 'C#', 'D', 'D#', 'E', 'F', 'F#', 'G', 'G#', 'A', 'A#', 'B'];

  const halfSteps = Math.round(12 * Math.log2(frequency / A4));

  const A4Index = notes.indexOf('A');
  let noteIndex = (A4Index + halfSteps) % 12;
  if (noteIndex < 0) noteIndex += 12;

  const octave = 4 + Math.floor((halfSteps + A4Index) / 12);
  
  return `${notes[noteIndex]}${octave}`;
};

export const MusicSlider = ({ isReadOnly, onFrequencyChange, frequency: externalFrequency, composerIsPlaying }) => {
  const [audioContext, setAudioContext] = useState(null);
  const [oscillator, setOscillator] = useState(null);
  const [gainNode, setGainNode] = useState(null);
  const sliderRef = useRef(null);
  const frequency = useRef(externalFrequency || 440);
  const isPressed = useRef(false);
  const isDragging = useRef(false);

  const initAudio = useCallback(() => {
    if (audioContext) return;

    try {
      const ctx = new (window.AudioContext || window.webkitAudioContext)();
      const osc = ctx.createOscillator();
      const gain = ctx.createGain();

      osc.type = 'sine';
      osc.frequency.setValueAtTime(frequency.current, ctx.currentTime);

      gain.gain.setValueAtTime(0.00001, ctx.currentTime);

      osc.connect(gain);
      gain.connect(ctx.destination);
      osc.start();

      setAudioContext(ctx);
      setOscillator(osc);
      setGainNode(gain);

      if (isReadOnly) {
        gain.gain.setValueAtTime(0.00001, ctx.currentTime);
      }
    } catch (error) {
      console.error("Audio initialization error:", error);
    }
  }, [audioContext, isReadOnly]);

  useEffect(() => {
    if (!isReadOnly || !externalFrequency) return;
    
    if (!audioContext) {
      initAudio();
      return;
    }

    if (oscillator && gainNode) {
      frequency.current = externalFrequency;
      oscillator.frequency.setValueAtTime(frequency.current, audioContext.currentTime);

      if (composerIsPlaying) {
        gainNode.gain.setTargetAtTime(0.5, audioContext.currentTime, 0.01);
      } else {
        gainNode.gain.setTargetAtTime(0.00001, audioContext.currentTime, 0.05);
      }
    }
  }, [externalFrequency, composerIsPlaying, oscillator, audioContext, gainNode, isReadOnly, initAudio]);

  useEffect(() => {
    if (isReadOnly) return;

    const calculateFrequency = (clientX) => {
      const slider = sliderRef.current;
      const rect = slider.getBoundingClientRect();
      const width = rect.width;

      let x = clientX - rect.left;
      if (clientX < rect.left) x = 0;
      if (clientX > rect.right) x = width;
      
      const percentage = x / width;
      return Math.round(220 + percentage * 880);
    };

    const handleMouseMove = (e) => {
      if (!isPressed.current) return;

      e.preventDefault();
      e.stopPropagation();

      const newFreq = calculateFrequency(e.clientX);
      frequency.current = newFreq;
      onFrequencyChange(newFreq, true);

      if (oscillator && gainNode) {
        oscillator.frequency.setValueAtTime(newFreq, audioContext.currentTime);
        gainNode.gain.setTargetAtTime(0.5, audioContext.currentTime, 0.01);
      }
    };

    const handleMouseDown = (e) => {
      const target = e.target;
      const isSlider = target.classList.contains('otamatone-neck');
      const isIndicator = target.classList.contains('frequency-indicator') || 
                         target.closest('.frequency-indicator');
      
      if (!isSlider && !isIndicator) return;

      e.preventDefault();
      e.stopPropagation();

      if (!audioContext) initAudio();
      isPressed.current = true;
      isDragging.current = true;
      
      if (gainNode) {
        gainNode.gain.setTargetAtTime(0.5, audioContext.currentTime, 0.01);
      }

      document.body.style.userSelect = 'none';
      const iframes = document.querySelectorAll('iframe');
      iframes.forEach(iframe => {
        iframe.style.pointerEvents = 'none';
      });
      
      onFrequencyChange(frequency.current, true);
      handleMouseMove(e);
    };

    const handleMouseUp = (e) => {
      if (!isPressed.current) return;
      
      e.preventDefault();
      e.stopPropagation();
      
      isPressed.current = false;
      isDragging.current = false;
      
      if (gainNode) {
        gainNode.gain.setTargetAtTime(0.00001, audioContext.currentTime, 0.05);
      }

      document.body.style.userSelect = '';
      const iframes = document.querySelectorAll('iframe');
      iframes.forEach(iframe => {
        iframe.style.pointerEvents = 'auto';
      });
      
      onFrequencyChange(frequency.current, false);
    };

    document.addEventListener('mousemove', handleMouseMove, true);
    document.addEventListener('mouseup', handleMouseUp, true);
    
    const slider = sliderRef.current;
    if (slider) {
      slider.addEventListener('mousedown', handleMouseDown, true);
    }

    return () => {
      document.removeEventListener('mousemove', handleMouseMove, true);
      document.removeEventListener('mouseup', handleMouseUp, true);
      if (slider) {
        slider.removeEventListener('mousedown', handleMouseDown, true);
      }
      document.body.style.userSelect = '';
      const iframes = document.querySelectorAll('iframe');
      iframes.forEach(iframe => {
        iframe.style.pointerEvents = 'auto';
      });
    };
  }, [isReadOnly, onFrequencyChange, audioContext, oscillator, gainNode, initAudio]);

  useEffect(() => {
    return () => {
      if (gainNode) gainNode.gain.setValueAtTime(0, audioContext.currentTime);
      if (oscillator) {
        try {
          oscillator.stop();
        } catch (e) {
          console.warn("Oscillator already stopped");
        }
      }
      if (audioContext) audioContext.close();
    };
  }, [oscillator, audioContext, gainNode]);

  const getFrequencyPosition = () => {
    const pos = ((frequency.current - 220) / 880) * 100;
    return `${Math.max(0, Math.min(pos, 100))}%`;
  };

  const getMouthSize = () => {
    const minSize = 30;
    const maxSize = 60;
    const freqRange = 880;
    const sizeRange = maxSize - minSize;
    const relativeFreq = frequency.current - 220;
    const size = minSize + (relativeFreq / freqRange) * sizeRange;
    return Math.max(minSize, Math.min(maxSize, size));
  };

  const shouldShowNoteMarker = () => {
    if (!isReadOnly) return isDragging.current;
    return composerIsPlaying;
  };

  if (isReadOnly && !composerIsPlaying) {
    return null;
  }

  return (
    <div className="otamatone-container">
      <div className="otamatone">
        <div className="otamatone-face">
          <div className="otamatone-eyes">
            <div className="eye left-eye"></div>
            <div className="eye right-eye"></div>
          </div>
          <div 
            className="otamatone-mouth"
            style={{
              width: `${getMouthSize()}px`,
              height: `${getMouthSize()}px`
            }}
          ></div>
        </div>
        {(!isReadOnly || composerIsPlaying) && (
          <div 
            ref={sliderRef}
            className={`otamatone-neck ${!isReadOnly ? 'interactive' : ''}`}
            style={{ cursor: isReadOnly ? 'default' : 'pointer' }}
          >
            <div
              className={`frequency-indicator ${isReadOnly ? 'read-only' : ''} ${composerIsPlaying ? 'active' : ''}`}
              style={{ left: getFrequencyPosition() }}
            >
              {shouldShowNoteMarker() && (
                <div className="note-marker">
                  {frequencyToNote(frequency.current)}
                </div>
              )}
            </div>
          </div>
        )}
      </div>
    </div>
  );
};